=begin
Multiple Face offset Version 2

Remus Knowles, 28/7/2009
modified by Andreas Eisenbarth, 27/4/2013

Problems? enquire here: http://forums.sketchucation.com/
=end

require 'sketchup'
require 'offset' # offset.rb by Rick Wilson, http://www.smustard.com/script/Offset

module RTK_AE

  def self.multi_face_scale_offset(percent=nil)

    unless percent.is_a?(Numeric)
      input = UI.inputbox(["relative offset"],[0],[],"Offset Distance")
      return unless input
      percent = input[0].to_f
    end

    model = Sketchup.active_model
    model.start_operation("Multiple Face Scale Offset")

    old_entities = model.active_entities.to_a
    sel = model.selection
    sel.grep(Sketchup::Face).each{|face|
      area = face.area
      edges = face.edges
      target_area = (1 + percent/100.0) * area
      circumfence = edges.inject(0){|sum, edge| sum + edge.length}
      # For rectangles
      if edges.length == 4 && edges.first.line[1].parallel?(edges[2].line[1]) && edges.first.line[1].perpendicular?(edges[1].line[1])
        l1 = edges.first.length
        l2 = edges[1].length
        # Solve: target_area = (l1 + 2*distance) * (l2 + 2*distance)
        distance = ( -2*(l1+l2) + ( 4*(l1+l2)**2 - 16*(l1*l2 - target_area) )**0.5 ) / 8.0
      else # Otherwise take a circle as reference
        ideal_circumfence = 2 * Math::PI * (area / Math::PI)**0.5
        ideal_length = (area / Math::PI)**0.5
        ideal_target_length = (target_area/Math::PI)**0.5
        # How much does the face deviate from the ideal shape?
        circumfence_ratio = (circumfence / ideal_circumfence)
        distance = (ideal_target_length - ideal_length) / circumfence_ratio
      end
      if defined?(TIG) && defined?(TIG::Smart_offset)
        TIG::Smart_offset.new(face, -distance)
      else
        face.offset(-distance)
      end
    }
    # Select new faces
    sel.clear
    sel.add(model.active_entities.to_a - old_entities)

    model.commit_operation

  end #method

end #module

unless file_loaded?(__FILE__)
  UI.menu("Plugins").add_item("Multi Face Scale Offset") {
    RTK_AE.multi_face_scale_offset
  }
  file_loaded(__FILE__)
end
